' *******************************************************************************************************
' *                                                                                                     *
' *     AiChip_Serial_002.spin                                                                          *
' *                                                                                                     *
' *******************************************************************************************************

CON

' .-----------------------------------------------------------------------------------------------------.
' |                                                                                                     |
' |     Characters for drawing logic analyser traces                                                    |
' |                                                                                                     |
' `-----------------------------------------------------------------------------------------------------'

  CHR_BIT_0_TO_0                = "_"
  CHR_BIT_0_TO_1                = "/"
  CHR_BIT_1_TO_0                = "^"
  CHR_BIT_1_TO_1                = "\"

' .-----------------------------------------------------------------------------------------------------.
' |                                                                                                     |
' |     Mode bit settings                                                                               |
' |                                                                                                     |
' `-----------------------------------------------------------------------------------------------------'

' .-----------------------------------------------------------------------------------------------------.
' |                                                                                                     |
' |     Standard ASCII characters                                                                       |
' |                                                                                                     |
' `-----------------------------------------------------------------------------------------------------'

  CR                            = $0D
  LF                            = $0A
  TAB                           = $09
  BKS                           = $08
                          
VAR

  byte rxMode
  byte txMode

  long column
  long row
    
' *******************************************************************************************************
' *                                                                                                     *
' *     Device Handling Routines                                                                        *
' *                                                                                                     *
' *******************************************************************************************************
                 
PUB OpenRx( setBasePin, setMode )

  if rxMode <> setMode

    CloseRx
    
    rxMode := setMode
 
  return rxMode

PUB CloseRx

  if rxMode
    rxMode := 0

PUB GetModeRx

  result := rxMode
                   
PUB OpenTx( setBasePin, setMode )
  
  if txMode <> setMode

    CloseTx
    Cls
    
    txMode := setMode
 
  return txMode

PUB CloseTx

  if txMode
    txMode := 0

PUB GetModeTx

  result := txMode
      
' *******************************************************************************************************
' *                                                                                                     *
' *     Text Handling Routines                                                                          *
' *                                                                                                     *
' *******************************************************************************************************

PUB TxByte( b )

  TxRawByte( b )
  
PUB TxRawByte( b )

  case b
    CR    : column := 0
    LF    : row++
    other : column++
    
  ' *****
  
PUB TxCrLf

  TxRawByte( CR )
  TxRawByte( LF )
  
PUB Cls

  column := row := 0
  
PUB Home
  
PUB TxTab

  TxRawByte( TAB )

PUB TxBackSpace

  TxRawByte( BKS )

' *******************************************************************************************************
' *                                                                                                     *
' *     Logic Analyser Trace Handling Routines                                                          *
' *                                                                                                     *
' *******************************************************************************************************

PUB GetLogicAnalyserBit(thisBit,nextBit)

  result := LookUpZ ( ( ( thisBit >> 30 ) & 2 ) | ( nextBit >> 31 ) : CHR_BIT_0_TO_0, CHR_BIT_0_TO_1, CHR_BIT_1_TO_0, CHR_BIT_1_TO_1 )

' *******************************************************************************************************
' *                                                                                                     *
' *     Receive Routines                                                                                *
' *                                                                                                     *
' *******************************************************************************************************

PUB RxIfAvailable : rxd

  rxd--

' *******************************************************************************************************
' *                                                                                                     *
' *     Attribute Handling for Video Drivers                                                            *
' *                                                                                                     *
' *******************************************************************************************************

PUB GetColumnCount
    
PUB GetRowCount
  
PUB GetCurrentColumn

  result := column

PUB GetCurrentRow

  result := row

' *******************************************************************************************************
' *                                                                                                     *
' *     End of AiChip_Serial_002.spin                                                                   *
' *                                                                                                     *
' *******************************************************************************************************